use crate::store::{gstore::*, store, State};
use gdk4::subclass::prelude::ObjectSubclassIsExt;
use gtk::prelude::*;

#[widget(@store extends gtk::Box)]
#[template(file = "nav.ui")]
pub struct NavHeaderButton {
    #[template_child]
    pub prev_button: TemplateChild<gtk::Button>,
    #[template_child]
    pub next_button: TemplateChild<gtk::Button>,

    #[selector(state.ui)]
    select_ui: (),
}

impl NavHeaderButton {
    pub fn new() -> Self {
        let o: Self = glib::Object::new();
        o
    }

    pub fn constructed(&self) {
        let prev_button = &self.imp().prev_button;
        let next_button = &self.imp().next_button;

        prev_button.connect_clicked(|_b| {
            dispatch!(crate::store::_GO_PREV);
        });
        next_button.connect_clicked(|_b| {
            dispatch!(crate::store::_GO_NEXT);
        });
    }

    fn select_ui(&self, state: &State) {
        let mobile = state.ui.mobile;
        let page = &state.ui.current_page;
        if page == crate::pages::WEEK_PAGE_NAME {
            self.next_button().set_visible(true);
            self.prev_button().set_visible(true);
        } else {
            self.next_button().set_visible(
                !mobile
                    && (page == crate::pages::DAY_PAGE_NAME
                        || page == crate::pages::MONTH_PAGE_NAME
                        || page == crate::pages::WEEK_PAGE_NAME),
            );
            self.prev_button().set_visible(
                !mobile
                    && (page == crate::pages::DAY_PAGE_NAME
                        || page == crate::pages::MONTH_PAGE_NAME
                        || page == crate::pages::WEEK_PAGE_NAME),
            );
        }
    }
}

impl Default for NavHeaderButton {
    fn default() -> Self {
        Self::new()
    }
}
