use crate::store::{gstore::DispatchWidget, store, State};
use gdk4::subclass::prelude::ObjectSubclassIsExt;
use gtk::prelude::*;

#[widget(@store extends gtk::Box)]
#[template(file = "new_event.ui")]
pub struct NewEventHeaderButton {
    #[template_child]
    pub button: TemplateChild<gtk::Button>,

    #[selector(state.ui.current_page, state.selection_slice)]
    select_ui_and_selection: (),
}

impl NewEventHeaderButton {
    pub fn new() -> Self {
        let o: Self = glib::Object::new();
        o
    }
    pub fn constructed(&self) {
        let button = &self.imp().button;
        button.connect_clicked(|b| {
            b.dispatch(crate::store::NEW_EVENT, None);
        });
    }

    fn select_ui_and_selection(&self, state: &State) {
        let current_page = &state.ui.current_page;
        let selection = &state.selection_slice;

        if current_page == "month" {
            self.button().set_visible(true);
            if selection.selection.is_some() {
                self.button().set_sensitive(true);
            } else {
                self.button().set_sensitive(false);
            }
        } else {
            self.button().set_visible(false)
        }
    }
}

impl Default for NewEventHeaderButton {
    fn default() -> Self {
        Self::new()
    }
}
