use adw::subclass::prelude::{ActionRowImpl, PreferencesRowImpl};
use gdk4::subclass::prelude::ObjectSubclassIsExt;
use gettextrs::gettext;
use gtk::{prelude::*, subclass::list_box_row::ListBoxRowImpl};
use libadwaita as adw;
use std::cell::Cell;

#[widget(extends adw::ActionRow)]
#[template(file = "remove_row.ui")]
pub struct RemoveRow {
    #[template_child]
    pub submit_button: TemplateChild<gtk::Button>,
    #[template_child]
    pub cancel_button: TemplateChild<gtk::Button>,

    #[property_bool]
    pub needs_submit: Cell<bool>,

    /// Wait time for submitting in seconds.
    #[property_u64]
    pub wait_time: Cell<u64>,

    //
    #[signal]
    submit: (),

    #[signal_handler(self activated)]
    on_click_row: (),
    #[signal_handler(cancel_button clicked)]
    on_cancel: (),
    #[signal_handler(submit_button clicked)]
    on_submit: (),
}

impl PreferencesRowImpl for imp::RemoveRow {}

impl ListBoxRowImpl for imp::RemoveRow {}

impl RemoveRow {
    pub fn new() -> Self {
        let o: Self = glib::Object::new();
        o.set_properties(&[
            //
            ("needs_submit", &false),
            ("wait_time", &2u64),
        ]);
        o
    }
    pub fn constructed(&self) {
        //
    }

    fn on_click_row(&self, _: gtk::Widget) {
        self.set_property("title", gettext("Are you sure?"));
        self.set_property("needs_submit", true);
        self.add_css_class("error");
    }
    fn on_cancel(&self, _: gtk::Button) {
        self.set_property("needs_submit", false);
        self.set_property("title", gettext("Remove"));
        self.remove_css_class("error");
    }
    fn on_submit(&self, b: gtk::Button) {
        self.on_cancel(b);
        self.emit_submit();
    }
}

impl Default for RemoveRow {
    fn default() -> Self {
        Self::new()
    }
}
