use gdk4::subclass::prelude::ObjectSubclassIsExt;
use std::cell::Cell;

use gtk::{ffi::GTK_STYLE_PROVIDER_PRIORITY_APPLICATION, prelude::*, CssProvider, GestureClick};

use crate::domain::get_text_color;

#[widget(extends gtk::Box)]
#[template(file = "week_event_tag.ui")]
pub struct WeekEventTag {
    #[property_string]
    pub label: Cell<String>,
    #[property_u64]
    pub num: Cell<u64>,
    #[property_bool]
    pub small: Cell<bool>,
    #[property_bool]
    pub transparent: Cell<bool>,

    #[template_child]
    pub color_box: TemplateChild<gtk::Box>,
    #[template_child]
    pub event_label_scrolled_window: TemplateChild<gtk::ScrolledWindow>,
    #[template_child]
    pub event_tag_label: TemplateChild<gtk::Label>,
    #[template_child]
    pub events_placeholder_num: TemplateChild<gtk::Label>,
    #[template_child]
    pub events_placeholder_label: TemplateChild<gtk::Label>,

    #[signal]
    click: (),
}

impl WeekEventTag {
    pub fn constructed(&self) {
        let _self = self;
        let gesture_click = GestureClick::builder().build();
        gesture_click.connect_released(glib::clone!(@weak _self => move |_, _, _, _| {
            _self.emit_click();
        }));
        self.add_controller(gesture_click);
    }

    pub fn new(label: String, num: u64, color: &str, small: bool) -> Self {
        let s: Self = glib::Object::new();
        s.set_properties(&[("label", &label), ("num", &num), ("small", &small)]);
        let color = color.to_string();
        s.connect_realize(move |s| s.realized(&color));
        if small {
            s.add_css_class("font-small");
        }
        if small {
            s.add_css_class("font-small");
        }
        s
    }

    fn realized(&self, color: &str) {
        let css = CssProvider::new();
        css.load_from_data(&format!(
            "* {{ background-color: {}; color: {};}}",
            color,
            get_text_color(color)
        ));
        self.color_box()
            .style_context()
            .add_provider(&css, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION as u32);
    }

    pub fn set_long(&self, long: bool) {
        if long {
            self.event_tag_label()
                .set_ellipsize(gdk4::pango::EllipsizeMode::None);
            self.event_tag_label().set_wrap(true);
            self.event_tag_label()
                .set_wrap_mode(gdk4::pango::WrapMode::WordChar);
            self.event_label_scrolled_window()
                .set_vscrollbar_policy(gtk::PolicyType::Always)
        }
    }

    pub fn set_folded(&self, folded: bool) {
        // self.event_tag_label().set_visible(!folded);
        self.event_label_scrolled_window().set_visible(!folded);
        self.events_placeholder_num().set_visible(folded);
        self.events_placeholder_label().set_visible(folded);
    }

    pub fn label(&self) -> String {
        self.property("label")
    }

    pub fn connect_click(&self, c: impl Fn(&Self) + 'static) {
        self.auto_cleanup_signal_handler(self._connect_click(c));
    }
}
