use gdk4::subclass::prelude::ObjectSubclassIsExt;
use gettextrs::gettext;
use glib::once_cell::sync::OnceCell;
use gtk::prelude::*;
use std::cell::Cell;

static NAMES: OnceCell<[String; 7]> = OnceCell::new();

#[widget(extends gtk::Box)]
#[template(file = "week_header_cell.ui")]
pub struct WeekHeaderCell {
    #[property_u64]
    pub index: Cell<u64>,
    #[property_u64]
    pub day: Cell<u64>,
    #[template_child]
    pub label: TemplateChild<gtk::Label>,
    #[template_child]
    pub sublabel: TemplateChild<gtk::Label>,
}

impl WeekHeaderCell {
    pub fn new(index: u64, day: u64) -> Self {
        let o: Self = glib::Object::new();
        o.set_properties(&[("index", &index), ("day", &day)]);
        o
    }
    pub fn constructed(&self) {
        self.connect_realize(|s| s.realized());
    }

    fn realized(&self) {
        let index = self.imp().index.clone().take();
        let day = self.imp().day.clone().take();

        let names = NAMES.get_or_init(|| {
            [
                gettext("Mo"),
                gettext("Tu"),
                gettext("We"),
                gettext("Th"),
                gettext("Fr"),
                gettext("Sa"),
                gettext("Su"),
            ]
        });
        self.label().set_label(&names[index as usize]);
        self.sublabel().set_label(&format!("{}", day));
    }
}
